#!/bin/sh

## Define function
# runiraf (iraf command) (output logfile) (output commandlist)
runiraf(){
  TMPLOG=$TMPPATH/tmp.log

  cat $1 >> $3
  cat $1 | cl > $TMPLOG
  cat $TMPLOG >> $2
  echo "" >> $2
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $2 #" 
   if [ $WFCAMDEBUG -eq 1 ]; then 
     \rm -r $TMPPATH
   fi
   exit 1
  fi
}

#Check Environmental Parameters
if [ `env | grep WFCAMDATA |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATA not defined!!"
 exit 1
fi

if [ `env | grep WFCAMDATE |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATE not defined!!"
 exit 1
fi

if [ `env | grep WFCAMLOG |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMLOG not defined!!"
 exit 1
fi

if [ `env | grep MASKRADIUS |gawk 'END{print(NR)}'` -eq 0 ]; then
 MASKRADIUS=6 
fi

MASKRADIUS=`echo $MASKRADIUS| gawk '{print(int($1/6*10))}'`
MASKRADIUS2=`echo $MASKRADIUS| gawk '{print(int($1*2))}'`

## Set DISPLAY Parameter
# WFCAMDISP=0; Display all images
#            1; Display minimum images
#            2; Display No image
if [ `env | grep WFCAMDISP |gawk 'END{print(NR)}'` -eq 0 ]; then
 WFCAMDISP=1 
fi

# Debug Parameter 
# WFCAMDEBUG=1; erase all temporary files 
#            0; leave all temporary files 
if [ `env | grep WFCAMDEBUG |gawk 'END{print(NR)}'` -eq 0 ]; then
WFCAMDEBUG=1
fi

## Check local command path
PATH=./bin/:$PATH
export PATH

BINPATH=`which fmagnify2`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fmagnify2 found. Check the PATH"
  exit 1
fi 
FMAGNIFY2=$BINPATH

BINPATH=`which fsubline`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fsubline found. Check the PATH"
  exit 1
fi 
FSUBLINE=$BINPATH

BINPATH=`which fwfcam`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fwfcam found. Check the PATH"
  exit 1
fi 
FWFCAM=$BINPATH

BINPATH=`which imcopy`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No imcopy found. Check the PATH"
  exit 1
fi 
IMCOPY=$BINPATH

# Check command arguments
if [ $# != 5 ]; then
    echo "Usage : "$0" (Header Name) (Detector ID) (Start No.) (No. of Frames) (No. of dither positions)"
    exit 1
fi

HEAD=$1
DETECTOR=$2
IS=$3
N=$4
NDITH=$5

# Set Temporary directory
TMPPATHBASE=/tmp/$USER
if [ ! -d $TMPPATHBASE ];then
  mkdir $TMPPATHBASE
fi

COMMANDHEAD=`date +%y%m%d%H%M%S`.objsubcombine2a.$HEAD

TMPPATH=$TMPPATHBASE/$COMMANDHEAD
if [ ! -d $TMPPATH ];then
 mkdir $TMPPATH
fi

# setup filenames
CLCOM=$TMPPATH/cl.com
CLLOG=log/$COMMANDHEAD.log
CLLOGCOM=`echo $CLLOG | sed 's/.log/.cl/g'`
TMPLOG=$TMPPATH/tmp.log

echo "$0 $1 $2 $3 $4 $5"
echo "# $0 $1 $2 $3 $4 $5" > $CLLOG

echo "# Log will be saved as "$CLLOG
echo "# IRAF commands will be saved as "$CLLOGCOM

# write environment parameters set to logfile
echo WFCAMDATA=$WFCAMDATA >> $CLLOG
echo WFCAMDATE=$WFCAMDATE >> $CLLOG
echo WFCAMLOG=$WFCAMLOG >> $CLLOG
echo WFCAMDEBUG=$WFCAMDEBUG >> $CLLOG

# Check Architecture
ARCH=`uname`
echo "# Running on "`uname -a`
echo "# Running on "`uname -a` >> $CLLOG

# Check IRAF Version
echo "logout" | cl 1> $TMPLOG
IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");
print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi
echo "# Start at $SECSTART" >> $CLLOG

# set filenames
SKYFLATBASE=flat.fits
SKYFLATEXPBASE=flatexp.pl
DARKBASE=dark${DETECTOR}.fits
BADPIXBASE=badpix${DETECTOR}.fits

SKYFLAT=${TMPPATH}/$SKYFLATBASE
SKYFLATEXP=${TMPPATH}/$SKYFLATEXPBASE
SKYFLATEXP2=${TMPPATH}/`echo $SKYFLATEXPBASE|sed 's/.pl$/.fits/g'`
DARK=${TMPPATH}/$DARKBASE
BADPIX=${TMPPATH}/$BADPIXBASE

FINAL=${HEAD}_${DETECTOR}.fits
FINALTBL=${HEAD}_${DETECTOR}.tbl
FINALOFFSET=${HEAD}_${DETECTOR}.offset
EXPMASK_FINAL=exp_${HEAD}_${DETECTOR}.fits
REJMASK_FINAL=rej_${HEAD}_${DETECTOR}.pl

DATAPATH=$WFCAMDATA/wfcam${DETECTOR}/${WFCAMDATE}

TDATAPATH=./tdata
LDATAPATH=./ldata
MASKPATH=./mask

if [ ! -d $LDATAPATH ];then
 mkdir $LDATAPATH
fi

if [ ! -d $TDATAPATH ];then
 mkdir $TDATAPATH
fi

# set detector ID specific information
case $DETECTOR in 
1) DETECTORC=w ; gawk -v is=$IS -v n=$NDITH 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($11/0.2,-$10/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
2) DETECTORC=x ; gawk -v is=$IS -v n=$NDITH 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,-$11/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
3) DETECTORC=y ; gawk -v is=$IS -v n=$NDITH 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$11/0.2,$10/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
4) DETECTORC=z ; gawk -v is=$IS -v n=$NDITH 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($10/0.2,$11/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
esac

# modify offset table (set minimum value of x and y coordinate to 0)
cat offset.tbl > $TMPPATH/tmp.tbl
gawk '{if(NR==1){x0=$1;y0=$2}x[NR]=$1;y[NR]=$2;if($1<x0)x0=$1;if($2<y0)y0=$2}END{for(i=1;i<=NR;i++)print(x[i]-x0,y[i]-y0)}' $TMPPATH/tmp.tbl > offset.tbl

# create local file table
gawk -v path=$DATAPATH -v date=$WFCAMDATE -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl
gawk -v path=$TDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > tdata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

if [ $WFCAMDISP -lt 2 ]; then
 echo ## Check if image viewer is present
 echo ## Check if image viewer is present >> $CLLOG

 TMPLOG=$TMPPATH/tmp.log
 cat << EOF | cl >$TMPLOG
  display ukirt.fits 1 fi+
  logout
EOF

 cat $TMPLOG >> $CLLOG
 
 if [ `grep ERROR $TMPLOG | gawk 'END{print(NR)}'` -gt 0 ]; then
  echo "# No image viewer is present."
  echo "# Set WFCAMDISP=2"
  echo "# No image viewer is present." >> $CLLOG
  echo "# Set WFCAMDISP=2" >> $CLLOG
  WFCAMDISP=2
 fi
fi

echo "## Copy Calibration Data to Temporary Path"
echo "## Copy Calibration Data to Temporary Path" >> $CLLOG

cat <<EOF > $CLCOM
 imcopy $SKYFLATBASE $SKYFLAT
 imcopy $SKYFLATEXPBASE $SKYFLATEXP
 imcopy $DARKBASE $DARK
 chpixtype $BADPIXBASE $BADPIX real
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "## Combine Frames at Each Position"
echo "## Combine Frames at Each Position" >> $CLLOG

# Create exposure map for skyflat frame
# Create exposure map for skyflat frame >> $CLLOG
HEDITTMP=$TMPPATH/hedit.tmp
cat <<EOF > $CLCOM
del $HEDITTMP
hedit $SKYFLATEXP EXP_TIME . > $HEDITTMP
logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

cat <<EOF > $CLCOM
imdel $SKYFLATEXP2
chpixtype $SKYFLATEXP $SKYFLATEXP2 real 
imarith $SKYFLATEXP2 * `gawk '{print($3)}' $HEDITTMP` $SKYFLATEXP2 
logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM


if [ -f combine.tbl ]; then
 \rm combine.tbl
fi
if [ -f tcombine.tbl ]; then
 \rm tcombine.tbl
fi
if [ -f expmask.tbl ]; then
\rm expmask.tbl
fi
if [ -f texpmask.tbl ]; then
\rm texpmask.tbl
fi
for masknumber in `gawk  -v n=$NDITH 'BEGIN{for(i=1;i<=n;i++)print(i)}'`;do
 masknumber2=`echo $masknumber | gawk '{printf("%03d",$1)}'`
 gawk -v m=$masknumber -v n=$NDITH '{if(((NR-1)%n +1)==m)print($0)}' ldata.tbl > $TMPPATH/tmp.${masknumber2}.tbl
 echo $LDATAPATH/${HEAD}_${DETECTOR}_$masknumber2.fits >> combine.tbl
 echo $LDATAPATH/exp_${HEAD}_${DETECTOR}_$masknumber2 >> expmask.tbl
 echo $TDATAPATH/texp_${HEAD}_${DETECTOR}_$masknumber2.fits >> texpmask.tbl
 echo $TDATAPATH/${HEAD}_${DETECTOR}_$masknumber2.fits >> tcombine.tbl
done

for masknumber in `gawk  '{print(NR)}' combine.tbl`;do
 masknumber2=`echo $masknumber | gawk '{printf("%03d",$1)}'`
 FRAME=`sed -n ${masknumber}p combine.tbl` 
 EXPMASK=`sed -n ${masknumber}p expmask.tbl` 
 TMPEXPFILE=$TMPPATH/tmpexp.txt
 echo "# Processing $FRAME "
 cat << EOF > $CLCOM
  imdel $FRAME,$EXPMASK
  imcombine @$TMPPATH/tmp.${masknumber2}.tbl $FRAME expmasks="$EXPMASK" combine=average reject=sigclip
  del $HEDITTMP
  hedit $EXPMASK EXP_TIME . > $HEDITTMP
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM 

 TMPFRAME1=$TMPPATH/tmpexp1.imh

 cat <<EOF > $CLCOM
  chpixtype $EXPMASK $EXPMASK real 
  imarith $EXPMASK * `gawk '{print($3)}' $HEDITTMP` $EXPMASK
  stsdas
  imdel $TMPFRAME1
  imcopy $EXPMASK $TMPFRAME1
  imdel $EXPMASK
  imcalc $TMPFRAME1,$SKYFLATEXP2 $EXPMASK "im1*im2/(im1+im2)"
  logout
EOF

echo  imdel $TMPFRAME1 >> $CLCOM
runiraf $CLCOM $CLLOG $CLLOGCOM


 if [ $WFCAMDISP -eq 0 ]; then
  cat <<EOF > $CLCOM
   display $FRAME 1 fi+
   logout
EOF
  runiraf $CLCOM $CLLOG $CLLOGCOM 
 fi

done

echo "## Subtract Background offsets"
echo "## Subtract Background offsets" >> $CLLOG

for dnumber in `gawk  '{print(NR)}' combine.tbl`;do
 masknumber=`gawk 'END{print(NR)}' offset.tbl`
  # if No of mask smaller than No of frames, take mod.
 dnumber2=`echo $dnumber $masknumber | gawk '{printf("%03d",(($1-1)%$2)+1)}'`
 FRAME=`sed -n ${dnumber}p combine.tbl` 
 MASKFILE=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.pl
 TEXTMASKFILE1=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.coo
 TEXTMASKFILE2=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.coo2

 IM_TMPMOD1=$TMPPATH/tmpmod1.imh
 IM_MOD1=$TMPPATH/mod1.fits
 IM_MOD2=$TMPPATH/mod2.imh
 IM_MOD3=$TMPPATH/mod3.imh
 IM_MOD4=$TMPPATH/mod4.imh
 
 echo "# Processing $FRAME using $TEXTMASKFILE1"
# Define where to carryout imsurfit for quad 2 and 3
 cat <<EOF > $TMPPATH/tmpsection.2
  101 1024 1 1024
  1 100 1 924
EOF
 cat <<EOF > $TMPPATH/tmpsection.4
  1 1024 1 924
  1 100 1 924
EOF
 cat << EOF > $CLCOM
  imdel ${IM_TMPMOD1}
  imcopy $FRAME ${IM_TMPMOD1}
  fixpix ${IM_TMPMOD1} $BADPIX 
  imedit ${IM_TMPMOD1} "" cursor=$TEXTMASKFILE1 radius=$MASKRADIUS buffer=5 width=5 default="b" display-
  imedit ${IM_TMPMOD1} "" cursor=$TEXTMASKFILE2 radius=$MASKRADIUS2 buffer=15 width=10 default="b" display-

  imdel ${IM_MOD1},${IM_MOD2},${IM_MOD3},${IM_MOD4}
  imcopy ${IM_TMPMOD1}[1:1024,1:1024] ${IM_MOD1}
  !./bin/fsubline ${IM_MOD1} ${IM_MOD1} x 5 3 mean
  imarith $FRAME[1:1024,1:1024] - ${IM_MOD1} ${IM_MOD2}
  imarith ${IM_TMPMOD1}[1:1024,1:1024] - ${IM_MOD1} ${IM_MOD3}
#new
  imdel ${IM_MOD1}
  imcopy ${IM_MOD3} ${IM_MOD1}
  !./bin/fsubline ${IM_MOD1} ${IM_MOD1} y 5 3 mean
  imarith ${IM_MOD2} - ${IM_MOD1} ${IM_MOD2}
  imarith ${IM_MOD3} - ${IM_MOD1} ${IM_MOD3}
#newend
  imsurfit ${IM_MOD3} ${IM_MOD4} 4 4 xmedian=5 ymedian=5 type_ou=fit border=10 cross_t=yes
  imarith ${IM_MOD2} - ${IM_MOD4} ${IM_MOD2}
  imcopy ${IM_MOD2} $FRAME[1:1024,1:1024]

  imdel ${IM_MOD1},${IM_MOD2},${IM_MOD3},${IM_MOD4}
  imcopy ${IM_TMPMOD1}[1:1024,1025:2048] ${IM_MOD1}
  !./bin/fsubline ${IM_MOD1} ${IM_MOD1} y 5 3 mean
  imarith $FRAME[1:1024,1025:2048] - ${IM_MOD1} ${IM_MOD2}
  imarith ${IM_TMPMOD1}[1:1024,1025:2048] - ${IM_MOD1} ${IM_MOD3}
#new
  imdel ${IM_MOD1}
  imcopy ${IM_MOD3} ${IM_MOD1}
  !./bin/fsubline ${IM_MOD1} ${IM_MOD1} x 5 3 mean
  imarith ${IM_MOD2} - ${IM_MOD1} ${IM_MOD2}
  imarith ${IM_MOD3} - ${IM_MOD1} ${IM_MOD3}
#newend
  imsurfit ${IM_MOD3} ${IM_MOD4} 4 4 xmedian=5 ymedian=5 type_ou=fit border=10 cross_t=yes sections=$TMPPATH/tmpsection.2 regions="sections"
  imarith ${IM_MOD2} - ${IM_MOD4} ${IM_MOD2}
  imcopy ${IM_MOD2} $FRAME[1:1024,1025:2048]
 
  imdel ${IM_MOD1},${IM_MOD2},${IM_MOD3},${IM_MOD4}
  imcopy ${IM_TMPMOD1}[1025:2048,1:1024] ${IM_MOD1}
  !./bin/fsubline ${IM_MOD1} ${IM_MOD1} y 5 3 mean
  imarith $FRAME[1025:2048,1:1024] - ${IM_MOD1} ${IM_MOD2}
  imarith ${IM_TMPMOD1}[1025:2048,1:1024] - ${IM_MOD1} ${IM_MOD3}
#new
  imdel ${IM_MOD1}
  imcopy ${IM_MOD3} ${IM_MOD1}
  !./bin/fsubline ${IM_MOD1} ${IM_MOD1} x 5 3 mean
  imarith ${IM_MOD2} - ${IM_MOD1} ${IM_MOD2}
  imarith ${IM_MOD3} - ${IM_MOD1} ${IM_MOD3}
#newend
  imsurfit ${IM_MOD3} ${IM_MOD4} 4 4 xmedian=5 ymedian=5 type_ou=fit border=10 cross_t=yes
  imarith ${IM_MOD2} - ${IM_MOD4} ${IM_MOD2}
  imcopy ${IM_MOD2} $FRAME[1025:2048,1:1024]

  imdel ${IM_MOD1},${IM_MOD2},${IM_MOD3},${IM_MOD4}
  imcopy ${IM_TMPMOD1}[1025:2048,1025:2048] ${IM_MOD1}
  !./bin/fsubline ${IM_MOD1} ${IM_MOD1} x 5 3 mean
  imarith $FRAME[1025:2048,1025:2048] - ${IM_MOD1} ${IM_MOD2}
  imarith ${IM_TMPMOD1}[1025:2048,1025:2048] - ${IM_MOD1} ${IM_MOD3}
#new
  imdel ${IM_MOD1}
  imcopy ${IM_MOD3} ${IM_MOD1}
  !./bin/fsubline ${IM_MOD1} ${IM_MOD1} y 5 3 mean
  imarith ${IM_MOD2} - ${IM_MOD1} ${IM_MOD2}
  imarith ${IM_MOD3} - ${IM_MOD1} ${IM_MOD3}
#newend
  imsurfit ${IM_MOD3} ${IM_MOD4} 4 4 xmedian=5 ymedian=5 type_ou=fit border=10 cross_t=yes sections=$TMPPATH/tmpsection.4 regions="sections"
#  imsurfit ${IM_MOD3} ${IM_MOD4} 4 4 xmedian=5 ymedian=5 type_ou=fit border=10 cross_t=yes
  imarith ${IM_MOD2} - ${IM_MOD4} ${IM_MOD2}
  imcopy ${IM_MOD2} $FRAME[1025:2048,1025:2048]

  imdel ${IM_TMPMOD1},${IM_MOD1},${IM_MOD2},${IM_MOD3},${IM_MOD4}
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM 

 if [ $WFCAMDISP -eq 0 ]; then
  cat <<EOF >$CLCOM
   display $FRAME 1 fi+
   logout
EOF
  runiraf $CLCOM $CLLOG $CLLOGCOM 
 fi

done

echo "## Divide pixels into 2x2"
echo "## Divide pixels into 2x2" >> $CLLOG
if [ -f $BADPIX ]; then
cat <<EOF > $CLCOM
imdel ${TMPPATH}/tmpbadpix.pl
!${FMAGNIFY2} $BADPIX ${TMPPATH}/tmpbadpix.fits
imarith ${TMPPATH}/tmpbadpix.fits * 4 ${TMPPATH}/tmpbadpix.fits
imcopy ${TMPPATH}/tmpbadpix.fits ${TMPPATH}/tmpbadpix.pl
logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM
fi

# divide data into 2x2 and add badpixel mask data to header
cat <<EOF > $CLCOM
imdel @tcombine.tbl
imdel @texpmask.tbl
chpixtype @expmask.tbl @texpmask.tbl real
imdel @expmask.tbl
EOF

for dnumber in `gawk  '{print(NR)}' combine.tbl`;do
 INFRAME=`sed -n ${dnumber}p combine.tbl` 
 OUTFRAME=`sed -n ${dnumber}p tcombine.tbl` 
 echo \!${FMAGNIFY2} $INFRAME $OUTFRAME >> $CLCOM

 if [ -f $BADPIX ]; then
  echo hedit $OUTFRAME bpm ${TMPPATH}/tmpbadpix.pl add+ ver- >> $CLCOM
 fi

 INFRAME=`sed -n ${dnumber}p texpmask.tbl` 
 OUTFRAME=`sed -n ${dnumber}p texpmask.tbl` 
 echo \!${FMAGNIFY2} $INFRAME $OUTFRAME >> $CLCOM
done


echo logout >> $CLCOM
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "## Shift and Add all frames"
echo "## Shift and Add all frames" >> $CLLOG

cat <<EOF > $CLCOM
 imdel $FINAL,$EXPMASK_FINAL,$REJMASK_FINAL
 imcombine @tcombine.tbl $FINAL rejmask="$REJMASK_FINAL" combine=average reject=sigclip offsets=offset.tbl masktype=goodvalue maskvalue=0
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "## Create Exposure Map"
echo "## Create Exposure Map" >> $CLLOG

TMPFRAME1=$TMPPATH/tmpexp1.imh
TMPFRAME2=$TMPPATH/tmpexp2.imh
TMPZERO=$TMPPATH/tmpexpzero.imh
TMPEXPFINAL=$TMPPATH/tmpexpfinal.imh

cat <<EOF > $CLCOM
 imdel $EXPMASK_FINAL,$TMPZERO
 imarith $FINAL * 0 $TMPZERO
 imcopy $TMPZERO $TMPEXPFINAL
EOF
for dnumber in `gawk  '{print(NR)}' texpmask.tbl`;do
 OFFSET=`sed -n ${dnumber}p offset.tbl` 
 INFRAME=`sed -n ${dnumber}p texpmask.tbl` 

 cat <<EOF >>$CLCOM
  imdel $TMPFRAME1,$TMPFRAME2
  imcopy $TMPZERO $TMPFRAME1
  imcopy $INFRAME $TMPFRAME1[1:4096,1:4096]
  imshift $TMPFRAME1 $TMPFRAME1 $OFFSET boundar=constant constan=0
  chpixtype ${REJMASK_FINAL}[,,${dnumber}] $TMPFRAME2 real
  imarith $TMPFRAME2 - 1.0 $TMPFRAME2
  imarith $TMPFRAME2 * -1.0 $TMPFRAME2
  imarith $TMPFRAME1 * $TMPFRAME2 $TMPFRAME1
  imarith $TMPEXPFINAL + $TMPFRAME1 $TMPEXPFINAL
EOF
done

cat <<EOF >> $CLCOM
imdel $TMPFRAME1,$TMPFRAME2,$TMPZERO
imcopy $TMPEXPFINAL $EXPMASK_FINAL
imdel $TMPEXPFINAL
logout

EOF
runiraf $CLCOM $CLLOG $CLLOGCOM


\cp tcombine.tbl $FINALTBL
\cp offset.tbl $FINALOFFSET

echo "# Final frame saved as $FINAL"
echo "# Frame table saved as $FINALTBL"
echo "# Offset table saved as $FINALOFFSET"
echo "# Exposure map saved as $EXPMASK_FINAL"
echo "# Rejected pixel map saved as $REJMASK_FINAL"

echo "# Final frame saved as $FINAL" >> $CLLOG
echo "# Frame table saved as $FINALTBL" >> $CLLOG
echo "# Offset table saved as $FINALOFFSET" >> $CLLOG
echo "# Exposure map saved as $EXPMASK_FINAL" >> $CLLOG
echo "# Rejected pixel map saved as $REJMASK_FINAL" >> $CLLOG

if [ $WFCAMDISP -lt 2 ]; then
 cat <<EOF > $CLCOM
  disp $FINAL 2 fi+
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM
fi

#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo "# End at $SECEND" >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}'
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}' >> $CLLOG

echo "# Log saved as $CLLOG"
echo "# IRAF commands saved as $CLLOGCOM"
if [ $WFCAMDEBUG -eq 1 ]; then 
  \rm -r $TMPPATH
else 
  echo "# See $TMPPATH for temporary files"
fi

exit 0




