#! /bin/sh

## Define function
# runiraf (iraf command) (output logfile) (output command log)
runiraf(){
  TMPLOG=$TMPPATH/tmp.log

  cat $1 >> $3
  cat $1 | cl > $TMPLOG
  cat $TMPLOG >> $2
  echo "" >> $2
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $2 #" 
   if [ $WFCAMDEBUG -eq 1 ]; then 
     \rm -r $TMPPATH
   fi
   exit 1
  fi
}

#Check Environmental Parameters
if [ `env | grep WFCAMDATA |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATA not defined!!"
 exit 1
fi

if [ `env | grep WFCAMDATE |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATE not defined!!"
 exit 1
fi

if [ `env | grep WFCAMLOG |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMLOG not defined!!"
 exit 1
fi

## Set DISPLAY Parameter
# WFCAMDISP=0; Display all images
#            1; Display minimum images
#            2; Display No images
if [ `env | grep WFCAMDISP |gawk 'END{print(NR)}'` -eq 0 ]; then
 WFCAMDISP=1 
fi

# Debug Parameter 
# WFCAMDEBUG=1; erase all temporary files 
#            0; leave all temporary files 
if [ `env | grep WFCAMDEBUG |gawk 'END{print(NR)}'` -eq 0 ]; then
WFCAMDEBUG=1
fi

## Check local command path
PATH=./bin/:$PATH
export PATH

BINPATH=`which fmagnify2`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fmagnify2 found. Check the PATH"
  exit 1
fi 
FMAGNIFY2=$BINPATH

BINPATH=`which fsubline`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fsubline found. Check the PATH"
  exit 1
fi 
FSUBLINE=$BINPATH

BINPATH=`which fwfcam`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fwfcam found. Check the PATH"
  exit 1
fi 
FWFCAM=$BINPATH

BINPATH=`which imcopy`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No imcopy found. Check the PATH"
  exit 1
fi 
IMCOPY=$BINPATH

#Check command arguments
if [ $# != 4 ]; then
    echo "Usage : "$0" (Header Name) (Detector ID) (Start No.) (No. of Frames)"
    exit 1
fi

HEAD=$1
DETECTOR=$2
IS=$3
N=$4

# Set Temporary directory
TMPPATHBASE=/tmp/$USER
if [ ! -d $TMPPATHBASE ];then
  mkdir $TMPPATHBASE
fi

COMMANDHEAD=`date +%y%m%d%H%M%S`.skyflat.$HEAD

TMPPATH=$TMPPATHBASE/$COMMANDHEAD
if [ ! -d $TMPPATH ];then
 mkdir $TMPPATH
fi

# setup filenames
CLCOM=$TMPPATH/cl.com
CLLOG=log/${COMMANDHEAD}.log
CLLOGCOM=`echo $CLLOG | sed 's/.log/.cl/g'`
TMPLOG=$TMPPATH/tmp.log

echo "$0 $1 $2 $3 $4"
echo "# $0 $1 $2 $3 $4" > $CLLOG

echo "# Log will be saved as "$CLLOG
echo "# IRAF commands will be saved as "$CLLOGCOM

# write environment parameters set to logfile
echo WFCAMDATA=$WFCAMDATA >> $CLLOG
echo WFCAMLOG=$WFCAMLOG >> $CLLOG
echo WFCAMDEBUG=$WFCAMDEBUG >> $CLLOG

# Check Architecture
ARCH=`uname`
echo "# Running on "`uname -a`
echo "# Running on "`uname -a` >> $CLLOG

# Check IRAF Version
echo "logout" | cl 1> $TMPLOG
IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");
print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi
echo "# Start at $SECSTART" >> $CLLOG

# set filenames
SKYFLAT=${HEAD}_skyflat${DETECTOR}.fits
SKY=${HEAD}_sky${DETECTOR}.fits

DARKBASE=dark${DETECTOR}.fits
DARK=${TMPPATH}/$DARKBASE

DATAPATH=$WFCAMDATA/wfcam${DETECTOR}/${WFCAMDATE}

TDATAPATH=./tdata
LDATAPATH=./ldata

if [ ! -d $LDATAPATH ];then
 mkdir $LDATAPATH
fi

if [ ! -d $TDATAPATH ];then
 mkdir $TDATAPATH
fi

# set detector ID specific information
case $DETECTOR in 
1) DETECTORC=w ;;
2) DETECTORC=x ;;
3) DETECTORC=y ;;
4) DETECTORC=z ;;
esac

# create local file table
gawk -v path=$DATAPATH -v date=$WFCAMDATE -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl
gawk -v path=$TDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > tdata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

if [ $WFCAMDISP -lt 2 ]; then
 echo ## Check if image viewer is present
 echo ## Check if image viewer is present >> $CLLOG
 
 TMPLOG=$TMPPATH/tmp.log
 cat << EOF | cl >$TMPLOG
  display ukirt.fits 1 fi+
  logout
EOF

 cat $TMPLOG >> $CLLOG

 if [ `grep ERROR $TMPLOG | gawk 'END{print(NR)}'` -gt 0 ]; then
  echo "# No image viewer is present."
  echo "# Set WFCAMDISP=2"
  echo "# No image viewer is present." >> $CLLOG
  echo "# Set WFCAMDISP=2" >> $CLLOG
  WFCAMDISP=2
 fi
fi
 
echo "## Copy Calibration Data to Temporary Path"
echo "## Copy Calibration Data to Temporary Path" >> $CLLOG

cat <<EOF > $CLCOM
# imcopy $FLATBASE $BADPIX
 imcopy $DARKBASE $DARK
# chpixtype $BADPIXBASE $BADPIX real
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "## Copy data to local disk"
echo "## Copy data to local disk" >> $CLLOG

cat << EOF > $CLCOM
 imdel @ldata.tbl | cl
 imdel @tdata.tbl | cl
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

paste data.tbl ldata.tbl | gawk -v dark=$DARK -v fwfcam=$FWFCAM '{printf("%s %s\"[1]\" %s %s\n",fwfcam,$1,$2,dark)}' >> $CLLOGCOM
paste data.tbl ldata.tbl | gawk -v dark=$DARK -v fwfcam=$FWFCAM '{printf("%s %s\"[1]\" %s %s\n",fwfcam,$1,$2,dark)}' | csh 1>> $CLLOG

echo "## Fix Header Information"
echo "## Fix Header Information" >> $CLLOG
gawk '{printf("hedit %s ctype1 RA ver-\nhedit %s ctype2 DEC ver-\n",$1,$1)}'  ldata.tbl > $CLCOM

# 1st imstat
cat << EOF >> $CLCOM
 del $TMPPATH/tmp.dat
 imstati @ldata-1.tbl format- > $TMPPATH/tmp.dat
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

# 2nd imstat
gawk -v tmppath=$TMPPATH 'BEGIN{printf("del %s/tmp.dat\n",tmppath)}{printf("imstati %s upper=%f lower=%f format- >> %s/tmp.dat\n",$1,$3+3*$4,$3-3*$4,tmppath)}' $TMPPATH/tmp.dat > $CLCOM
echo logout >>$CLCOM
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "## Uniform background level"
echo "## Uniform Background level" >> $CLLOG
gawk '{s+=$3;a[NR]=$3}END{s=s/NR;for(i=1;i<=NR;i++)print(a[i]/s)}' $TMPPATH/tmp.dat > $TMPPATH/tmp2.dat

cat << EOF > $CLCOM
 imarith @ldata.tbl / @${TMPPATH}/tmp2.dat @ldata.tbl 
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

echo "## Combine and create a sky frame"
echo "## Combine and create a sky frame" >> $CLLOG
cat <<EOF > $CLCOM
 imdel $SKY
 imcombine @ldata.tbl $SKY combine=median reject=sigclip
 del $TMPPATH/tmp.dat
 imstati $SKY format- > $TMPPATH/tmp.dat
logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 


IMSTATUPPER=`gawk '{print($3+$4*3)}' $TMPPATH/tmp.dat`
IMSTATLOWER=`gawk '{print($3-$4*3)}' $TMPPATH/tmp.dat`

echo "## Normalize to create a skyflat"
echo "## Normalize to create a skyflat" >> $CLLOG
cat <<EOF > $CLCOM
 del $TMPPATH/tmp.dat
 imstati $SKY format- fields=mean upper=$IMSTATUPPER lower=$IMSTATLOWER > $TMPPATH/tmp.dat
 imdel $SKYFLAT
 imarith $SKY / @$TMPPATH/tmp.dat $SKYFLAT
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

echo "## Fix bad pixels"
echo "## Fix bad pixels" >> $CLLOG
cat << EOF > $CLCOM
 imdel $TMPPATH/tmp.fits
 imcopy $SKYFLAT $TMPPATH/tmp.fits
 imreplace $TMPPATH/tmp.fits -1 upper=0.01 
 imreplace $TMPPATH/tmp.fits 0 lower=0
 imreplace $TMPPATH/tmp.fits 1 upper=-0.5
 fixpix $SKYFLAT $TMPPATH/tmp.fits
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

if [ $WFCAMDISP -lt 2 ]; then
 cat <<EOF > $CLCOM
  disp $SKYFLAT 1 fi+
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM
fi

echo "# Sky frame saved as $SKY"
echo "# Skyflat frame saved as $SKYFLAT"

#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo "# End at $SECEND" >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}'
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}' >> $CLLOG

echo "# Log saved as $CLLOG"
echo "# IRAF commands saved as $CLLOGCOM"
if [ $WFCAMDEBUG -eq 1 ]; then 
  \rm -r $TMPPATH
else 
  echo "# See $TMPPATH for temporary files"
fi

exit 0
